/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  Address: Wilhelm-Maybach-Str. 24, 68766 Hockenheim, Germany
 *  E-mail: info@opengpi.org 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** The KeyInput class offers graphical objects as input elements for the parameters. 
 * A KeyInput is presented as a box with fixed size (width 400 pixels, height 70 pixels), which contains the input element.
 * The input element can either be a text line, a drop-down-menu or an integer spinbox.
 * The keyname appears as the title of the box.
 */


#ifndef KEYINPUT_H
#define KEYINPUT_H

#include <QtGui>

class KeyInput : public QWidget
{
private:
	Q_OBJECT
	QVariant value(); /**< Returns the current value of the input element as a QVariant. */


signals:

	void valueChanged(QVariant,QModelIndex); /**< This signal is emitted, when the user changes the value of the input element.
						      * It passes the new value as first argument, and the index corresponding to the key
						      * as second argument.*/
	void showMyInfo(QString,QString); /**< This signal is emitted, when the user points with the mouse inside the KeyInput widget.
				       * It passes the keyname as first argument, and the keyinfo as second argument.*/

public:

	/**
   * Constructs and initializes a KeyInput.
   * \param parent The parent widget.
   * \param data Contains the data used to initialize the KeyInput.
   * \param index Refers to the Key in the TreeModel this KeyInput belongs to.
	 */
	KeyInput(QWidget *parent, QVariant data, QModelIndex index);

	/**
	 * Constructs an empty (uninitialized) KeyInput.
	 * \param parent The parent widget
	 */
	KeyInput(QWidget *parent);

	~KeyInput(); /**< The destructor. */

	QModelIndex index(); /**< Returns the index corresponding to the key.*/

	/**
	 * Changes the input element data.
	 * Determins the type of the QVariant v.
	 * If v is a QMap<QString,QVariant>, the input element's value and items in the drop-down-menu
	 * or the min and max of the spinbox are changed, depending of which type the input element is.
	 * Else, only the current value of the input element is changed.
	 * \param v Contains the data for the new settings, either just a value or a QMap<QString,QVariant>
	 * that defines also new items for the drop-down-menu or min and max for the spinbox.
	 */
	void changeData(QVariant v);

private:

	QGroupBox *box;
	QLineEdit *lineEdit;
	QComboBox *comboBox;
	QSpinBox *spinBox;
	QModelIndex myIndex;
	QString myInfo;
	QPushButton *browseButton;
	int t;
	void mouseMoveEvent(QMouseEvent * event);
	QMap<QString,QVariant> map;

private slots:

	void changeValue(QString s);
	void changeValue(int i);
	void browseButtonClicked();
};

#endif
