/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  Address: Wilhelm-Maybach-Str. 24, 68766 Hockenheim, Germany
 *  E-mail: info@opengpi.org 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "opengpigui.h"

OpenGPIGUI::OpenGPIGUI(QWidget *parent,Qt::WindowFlags flags)
: QMainWindow(parent,flags)
{
	resize(775,600);

	CentralWidget = new QWidget(this);
	setCentralWidget(CentralWidget);

	createMenu();
	createTree();
	createWizard();

	Layout = new QGridLayout(CentralWidget);
	Layout->setContentsMargins(0,0,0,0);
	Layout->setAlignment(0);
	Layout->setSpacing(0);

	Layout->addWidget(Tree,0,0,1,1);
	Layout->addWidget(Wizard,0,1,1,1);

	CentralWidget->setLayout(Layout);
}

OpenGPIGUI::~OpenGPIGUI(){}

void OpenGPIGUI::createMenu()
{
	LoadDescriptionAction = new QAction(tr("Load Description File"), this);
	LoadParamAction = new QAction(tr("Load Parameters"), this);
	SaveParamAction = new QAction(tr("Save Parameters"), this);
    QuitAction = new QAction(tr("Quit"), this);

	LoadDescriptionAction->setEnabled(true);
	LoadParamAction->setEnabled(false);
	SaveParamAction->setEnabled(false);
    QuitAction->setEnabled(true);


	connect(LoadDescriptionAction, SIGNAL(triggered()), this, SLOT(enableOtherActions()));
	connect(LoadDescriptionAction, SIGNAL(triggered()), this, SLOT(getDescriptionFileName()));
	connect(LoadParamAction, SIGNAL(triggered()), this, SLOT(getParamFileName()));
	connect(SaveParamAction, SIGNAL(triggered()), this, SLOT(getOutFileName()));
    connect(QuitAction, SIGNAL(triggered()), this, SLOT(close()));

	MenuBar = new QMenuBar(this);
	FileMenu = new QMenu(this);
	setMenuBar(MenuBar);
	FileMenu = MenuBar->addMenu("File");
	FileMenu->setEnabled(true);

	FileMenu->addAction(LoadDescriptionAction);
	FileMenu->addSeparator();
	FileMenu->addAction(LoadParamAction);
	FileMenu->addSeparator();
	FileMenu->addAction(SaveParamAction);
    FileMenu->addAction(QuitAction);
}

void OpenGPIGUI::createTree()
{
	Tree = new OpenGPITree(this);
	Tree->setMinimumWidth(350);
	Tree->setMinimumHeight(200);
	Tree->setFrameShape(QFrame::Box);
	connect(Tree,SIGNAL(checkAllBlocks()),this,SIGNAL(checkAllBlocks()));
}

void OpenGPIGUI::createWizard()
{
	Wizard = new OpenGPIWizard(this);
	connect(Wizard,SIGNAL(sendInFileName(QString)),this,SIGNAL(sendDescriptionFileName(QString)));
	connect(Wizard,SIGNAL(getNextKeys(QModelIndex)),this,SIGNAL(getNextKeys(QModelIndex)));
	connect(Wizard,SIGNAL(getPreviousKeys(QModelIndex)),this,SIGNAL(getPreviousKeys(QModelIndex)));
	connect(Wizard,SIGNAL(getKeys(QModelIndex)),this,SIGNAL(getKeys(QModelIndex)));
	connect(Wizard,SIGNAL(valueChanged(QVariant,QModelIndex)),this,SIGNAL(valueChanged(QVariant,QModelIndex)));
	connect(Wizard->button(QWizard::CancelButton),SIGNAL(clicked()),this,SLOT(expertMode()));
	connect(Wizard,SIGNAL(sendOutFileName(QString)),this,SIGNAL(sendOutFileName(QString)));
	connect(Wizard->button(QWizard::FinishButton),SIGNAL(clicked()),this,SLOT(showFinishDialog()));
	connect(this,SIGNAL(dataChanged(QModelIndex,QVariant)),Wizard,SLOT(changeData(QModelIndex,QVariant)));
	connect(Wizard,SIGNAL(getCurrentSettings()),this,SIGNAL(getCurrentSettings()));
	connect(this,SIGNAL(sendCurrentSettings(QTreeWidgetItem*)),Wizard,SLOT(showCurrentSettingsDialog(QTreeWidgetItem*)));
	connect(this,SIGNAL(showInfo(QString)),Wizard,SLOT(showInfo(QString)));
	connect(this,SIGNAL(giveDescriptionFileName(QString)),Wizard,SLOT(setInFileName(QString)));
	connect(Wizard,SIGNAL(sendKeypageDisplayed(QList<QModelIndex>)),Tree,SLOT(selectIndex(QList<QModelIndex>)));
}

void OpenGPIGUI::clearGui()
{
	Wizard->clear();
}

void OpenGPIGUI::enableOtherActions()
{
	LoadParamAction->setEnabled(true);
	SaveParamAction->setEnabled(true);
}

void OpenGPIGUI::getDescriptionFileName()
{
	DescriptionFileName.clear();
	DescriptionFileName = QFileDialog::getOpenFileName(this, tr("Load Description File"),"./",tr("XML files (*.xml)"));
	clearGui();
	Wizard->restart();
	Wizard->next();
	emit giveDescriptionFileName(DescriptionFileName);
}

void OpenGPIGUI::getParamFileName()
{
	ParamFileName.clear();
	ParamFileName = QFileDialog::getOpenFileName(this, tr("Load Parameter File"),"./",tr("XML files (*.xml)"));
	emit sendParamFileName(ParamFileName);
}

void OpenGPIGUI::getOutFileName()
{
	OutFileName.clear();
	OutFileName = QFileDialog::getSaveFileName(this, tr("Save current settings"),"./",tr("XML files (*.xml)"));
	if(!OutFileName.isEmpty())
	{
		emit sendOutFileName(OutFileName);
		showFinishDialog();
	}
}

void OpenGPIGUI::receiveModel(QAbstractItemModel* model)
{
	Tree->setModel(model);
	Tree->setProperties();
	enableOtherActions();
	emit getFirstKeys();
	Wizard->next();

	connect(Tree,SIGNAL(clicked(QModelIndex)),this,SIGNAL(getKeys(QModelIndex)));
}

void OpenGPIGUI::receiveNextKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	Wizard->receiveNextKeys(data,index);
}

void OpenGPIGUI::receivePreviousKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	Wizard->receivePreviousKeys(data,index);
}

void OpenGPIGUI::receiveKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	Wizard->receiveKeys(data,index);
}

void OpenGPIGUI::showFinishDialog()
{
	finishDialog = new OpenGPIDialog(this);
	int r = finishDialog->exec();
	switch(r)
	{
	case 0:
		Wizard->restart();
		Wizard->next();
		break;

	case 1:
		emit getFirstKeys();
		if(Wizard->currentId() == 3)
			Wizard->back();

		break;

	case 2:
		close();
		break;

	default:
		break;
	}
}
