/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  Address: Wilhelm-Maybach-Str. 24, 68766 Hockenheim, Germany
 *  E-mail: info@opengpi.org 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "opengpiwizard.h"

OpenGPIWizard::OpenGPIWizard(QWidget *parent)
: QWizard(parent)
{
	startpage = new StartPage(this);
	loadpage = new LoadPage(this);
	keypage = new KeyPage(this);
	finalpage = new FinalPage(this);



	setPage(Page_Start,startpage);
	setPage(Page_Load,loadpage);
	setPage(Page_Keys,keypage);
	setPage(Page_Final,finalpage);

	setOption(QWizard::NoCancelButton);
	setOption(QWizard::HaveCustomButton1);
	setButtonText(QWizard::CustomButton1,"Current Settings");

	QList<QWizard::WizardButton> list;
	list << QWizard::CustomButton1 << QWizard::Stretch << QWizard::BackButton << QWizard::NextButton << QWizard::FinishButton;
	setButtonLayout(list);
	button(QWizard::CustomButton1)->setEnabled(false);

	disconnect(button(QWizard::NextButton),SIGNAL(clicked()),0,0);
	disconnect(button(QWizard::BackButton),SIGNAL(clicked()),0,0);
	disconnect(button(QWizard::FinishButton),SIGNAL(clicked()),0,0);

	connect(button(QWizard::NextButton),SIGNAL(clicked()),this,SLOT(checkNext()));
	connect(button(QWizard::BackButton),SIGNAL(clicked()),this,SLOT(checkBack()));
	connect(button(QWizard::CustomButton1),SIGNAL(clicked()),this,SIGNAL(getCurrentSettings()));

	connect(button(QWizard::FinishButton),SIGNAL(clicked()),this,SLOT(getOutFileName()));

	connect(loadpage,SIGNAL(inFileNameIsOK(bool)),this,SLOT(enableNextButton(bool)));
	connect(keypage,SIGNAL(valueChanged(QVariant,QModelIndex)),this,SIGNAL(valueChanged(QVariant,QModelIndex)));
	connect(finalpage,SIGNAL(outFileNameIsOK(bool)),this,SLOT(enableFinishButton(bool)));
}

OpenGPIWizard::~OpenGPIWizard(){}

void OpenGPIWizard::showInfo(QString s)
{
	keypage->setInfo(s);
}

void OpenGPIWizard::clear()
{
	keypage->clear();
}

void OpenGPIWizard::checkNext()
{
	switch(currentId())
	{
	case Page_Start:
		next();
		loadpage->checkInFileName();
		break;

	case Page_Load:
		emit sendInFileName(loadpage->currentText());
		button(QWizard::CustomButton1)->setEnabled(true);
		break;

	case Page_Keys:
		keypage->setInfo("","");
		emit getNextKeys(currentIndex.value(0));
		break;

	default:
		break;
	}
}

void OpenGPIWizard::checkBack()
{
	switch(currentId())
	{
	case Page_Load:
		back();
		break;

	case Page_Keys:
		keypage->setInfo("","");
		emit getPreviousKeys(currentIndex.value(0));
		break;

	case Page_Final:
		back();
		emit sendKeypageDisplayed(currentIndex);
		break;

	default:
		break;
	}
}

void OpenGPIWizard::enableNextButton(bool b)
{
	button(QWizard::NextButton)->setEnabled(b);
}

void OpenGPIWizard::enableFinishButton(bool b)
{
	button(QWizard::FinishButton)->setEnabled(b);
}

void OpenGPIWizard::setInFileName(QString s)
{
	loadpage->setText(s);
	checkNext();
}

void OpenGPIWizard::receiveNextKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	emit sendKeypageDisplayed(index);
	if(index.isEmpty())
	{
		next();
		finalpage->checkOutFileName();
	}
	else
	{
		currentIndex = index;
		keypage->setKeys(data,index);
	}
}

void OpenGPIWizard::receiveKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	currentIndex = index;
	keypage->setKeys(data,index);
	switch(currentId())
	{
	case Page_Start:
		next(); // Next Page is Page_Load
	case Page_Load:
		next();
		break;

	case Page_Final:
		back();
		break;

	default:
		break;
	}
}

void OpenGPIWizard::receivePreviousKeys(QList<QVariant> data,QList<QModelIndex> index)
{
	emit sendKeypageDisplayed(index);
	if(index.isEmpty())
	{
		back();
		button(QWizard::CustomButton1)->setEnabled(false);
	}
	else
	{
		currentIndex = index;
		keypage->setKeys(data,index);
	}
}

void OpenGPIWizard::changeData(QModelIndex i,QVariant v)
{
	if(currentIndex.contains(i))
		keypage->changeData(i,v);
}

void OpenGPIWizard::getOutFileName()
{
	emit sendOutFileName(finalpage->getText());
}

void OpenGPIWizard::showCurrentSettingsDialog(QTreeWidgetItem* root)
{
	csd = new CurrentSettingsDialog(this,root);
	csd->exec();
}
