/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  Address: Wilhelm-Maybach-Str. 24, 68766 Hockenheim, Germany
 *  E-mail: info@opengpi.org 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * The OpenGPIWizard provides four wizard pages: a StartPage, a LoadPage, a KeyPage and a FinalPage.
 * It has a back button and a next button to navigate through the wizard pages. The current settings button displays a user dialog
 * with a representation of the current key values.
 */


#ifndef OPENGPIWIZARD
#define OPENGPIWIZARD

#include <QtGui>
#include "startpage.h"
#include "loadpage.h"
#include "keypage.h"
#include "finalpage.h"
#include "currentsettingsdialog.h"

class OpenGPIWizard : public QWizard
{
	Q_OBJECT

public:
	/**
	 * Constructs a OpenGPIWizard with a StartPage, a LoadPage, a KeyPage and a FinalPage.
	 * \param parent The parent widget.
	 */
	OpenGPIWizard(QWidget *parent);

	/**
	 * The destructor.
	 */
	~OpenGPIWizard();

	void clear(); /**< Clears the KeyPage.*/

public slots:

	/**
	 * Puts the directory of the input file name (description or saved parameters) into the text line of the LoadPage and loads the file.
	 * \param s The directory of the file as a string.
	 */
	void setInFileName(QString s);

	/**
	 * Receives the data of the next keys that are to be displayed on the KeyPage (e.g. after the signal getNextKeys(QModelIndex) has been emitted). Advances to the FinalPage if the data is empty.
	 * \param data List of data of the keys to be displayed.
	 * \param index List of indexes of the keys to be displayed.
	 */
	void receiveNextKeys(QList<QVariant> data, QList<QModelIndex> index);
	
	/**
	 * Receives the data of the keys that are to be displayed on the KeyPage. This happens when a block in the TreeView is clicked.
	 * \param data List of data of the keys to be displayed.
	 * \param index List of indexes of the keys to be displayed.
	 */
	void receiveKeys(QList<QVariant> data, QList<QModelIndex> index);

	/**
	 * Receives the data of the previous keys that are to be displayed on the KeyPage (e.g. after the signal getPreviousKeys(QModelIndex) has been emitted). Returns to the LoadPage if the data is empty.
	 * \param data List of data of the keys to be displayed.
	 * \param index List of indexes of thekeys to be displayed.
	 */
	void receivePreviousKeys(QList<QVariant> data, QList<QModelIndex> index);

	/**
	 * Changes the data of the KeyInput for the key with index i, if it is currently displayed on the KeyPage.
	 * \param i Index of the key whose data has been changed.
	 * \param v New data.
	 */
	void changeData(QModelIndex i,QVariant v);

	/**
	 * Displays a user dialog that presents the current parameter settings as a tree diagram of blocks and keys.
	 * \param root The (invisible) root item in the tree diagram to be displayed.
	 */
	void showCurrentSettingsDialog(QTreeWidgetItem* root);

	/**
	 * Shows a warning message in the InfoBox of the KeyPage.
	 * \param s The warning message.
	 */
	void showInfo(QString s);

signals:

	void sendInFileName(QString); /**< This signal is emitted, if a valid file name (description file or file with saved parameters) has been specified (via the LoadPage or the menu). It passes the directory of the file as a string.*/
	void sendOutFileName(QString); /**< This signal is emitted, if a valid file name and directory has been specified to save the current settings.*/
	void getNextKeys(QModelIndex); /**< This signal is emitted, if the user presses the next button on the KeyPage. It passes the index of the first key that is currently displayed (with the intention to get the keys of the next block).*/
	void getPreviousKeys(QModelIndex); /**< This signal is emitted, if the user presses the next button on the KeyPage. It passes the index of the first key that is currently displayed (with the intention to get the keys of the previous block).*/
	void getCurrentSettings(); /**< This signal is emitted, if the user presses the current settings button.*/
	void valueChanged(QVariant,QModelIndex); /**< This signal is forwarding the signal valueChanged(QVariant,QModelIndex) of the KeyPage.*/
	void sendKeypageDisplayed(QList<QModelIndex>); /**< Send the index of the
													 current Keypage, it there
													 is one being displayed */

private:

	StartPage *startpage;
	LoadPage *loadpage;
	KeyPage *keypage;
	FinalPage *finalpage;

	enum { Page_Start, Page_Load, Page_Keys, Page_Final };

	CurrentSettingsDialog *csd;

	QList<QModelIndex> currentIndex;

private slots:

	void checkNext();
	void checkBack();

	void enableNextButton(bool b);
	void enableFinishButton(bool b);

	void getOutFileName();


};

#endif
