/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  Address: Wilhelm-Maybach-Str. 24, 68766 Hockenheim, Germany
 *  E-mail: info@opengpi.org 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>

#include "treeitem.h"
#include "treemodel.h"

TreeModel::TreeModel(QObject *parent)
: QAbstractItemModel(parent)
{
	QList<QVariant> rootData;
	rootData << "Block Structure" << "Number of Keys";
	rootItem = new TreeItem(rootData);
}

TreeModel::~TreeModel()
{
	delete rootItem;
}

QVariant TreeModel::data(const QModelIndex &index, int role) const
{
	if(!index.isValid())
		return QVariant();

	if(role == Qt::ForegroundRole) {
		if( !isDataValid(index)) {
			return QVariant(QColor(Qt::red));
		}
		return QVariant();
	}

	if(role != Qt::DisplayRole)
		return QVariant();

	TreeItem *item = static_cast<TreeItem*>(index.internalPointer());

	return item->data(index.column());
}

bool TreeModel::isDataValid(const QModelIndex &index) const
{
	TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
	QMap<QString, QVariant > itemdata;

	if(index.column() <= 2) {
		// this is a block
		for(int i = 4; i < item->columnCount(); i++) {
			itemdata = item->data(i).toMap();
			if(
					(itemdata.contains("min") && itemdata.value("min").toInt() > itemdata.value("value").toInt()) ||
					(itemdata.contains("max") && itemdata.value("max").toInt() < itemdata.value("value").toInt()) ||
					( itemdata.contains("currentlist") && ! itemdata.value("currentlist").toMap().contains(itemdata.value("value").toString() ) )
			  ){
				return false;
			}
		}
		return true;

	}

	itemdata = item->data(index.column()).toMap();

	if(
			(itemdata.contains("min") && itemdata.value("min").toInt() > itemdata.value("value").toInt()) ||
			(itemdata.contains("max") && itemdata.value("max").toInt() < itemdata.value("value").toInt()) ||
			( itemdata.contains("currentlist") && ! itemdata.value("currentlist").toMap().values().contains(itemdata.value("value").toString() ) )
	  ) {
		return false;
	}
	return true;
}

QVariant TreeModel::headerData(int section, Qt::Orientation orientation,
		int role) const
{
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
		return rootItem->data(section);

	return QVariant();
}

QModelIndex TreeModel::index(int row, int column, const QModelIndex &parent)
	const
{
	if (!hasIndex(row, column, parent))
		return QModelIndex();

	TreeItem *parentItem;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem*>(parent.internalPointer());

	TreeItem *childItem = parentItem->child(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex TreeModel::parent(const QModelIndex &index) const
{
	if (!index.isValid())
		return QModelIndex();

	TreeItem *childItem = static_cast<TreeItem*>(index.internalPointer());
	TreeItem *parentItem = childItem->parent();

	if (parentItem == rootItem)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

int TreeModel::rowCount(const QModelIndex &parent) const
{
	TreeItem *parentItem;
	if (parent.column() > 0)
		return 0;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem*>(parent.internalPointer());

	return parentItem->childCount();
}

int TreeModel::columnCount(const QModelIndex &parent) const
{
	if (parent.isValid())
		return static_cast<TreeItem*>(parent.internalPointer())->columnCount();
	else
		return rootItem->columnCount();
}

bool TreeModel::setDataNoDeps(const QModelIndex &index, const QVariant &value, int role)
{
	if (index.isValid())
	{
		TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
		QString block, key;
		QMap<QString,QVariant> map = item->data(index.column()).toMap();
		block = item->data(3).toString();
		key = map.value("name").toString();

		if(index.column() < 4)
			item->itemData.replace(index.column(),value);
		else
		{
			if(value.type() == QVariant::Map)
				item->itemData.replace(index.column(),value);
			else
			{
				map.insert("value",value);
				item->itemData.replace(index.column(),map);
			}
		}
		emit dataChanged(index, value);
		return true;
	}
	return false;
}
bool TreeModel::setData(const QModelIndex &index, const QVariant &value, int role)
{

	if(setDataNoDeps(index, value, role)) {

		TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
		QString block, key;
		QMap<QString,QVariant> map = item->data(index.column()).toMap();
		block = item->data(3).toString();
		key = map.value("name").toString();

		checkDep(block,key);
		return true;
	}
	return false;
}

bool TreeModel::setBlockEnabled(QModelIndex i, bool b)
{
	if (i.isValid())
	{
		TreeItem* item = static_cast<TreeItem*>(i.internalPointer());
		item->itemData.replace(2,b);
		emit blockChanged(i);
		return true;
	}

	return false;
}

QList<QVariant> TreeModel::keyDataList(QModelIndex index)
{
	QList<QVariant> list;

	if(!index.isValid())
		return list;


	TreeItem *item = static_cast<TreeItem*>(index.internalPointer());

	list.append(item->data(0)); // display name of block first
	for(int i = 4; i < item->columnCount(); i++)
		list.append(item->data(i));
	return list;
}

QList<QModelIndex> TreeModel::keyIndexList(QModelIndex index)
{
	QList<QModelIndex> list;

	if(!index.isValid())
		return list;

	TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
	list.append(createIndex(index.row(),0,item));
	for(int i = 4; i <= item->columnCount(); i++) {
		list.append(createIndex(index.row(),i,item));
	}
	return list;
}

void TreeModel::goIntoBlock(QDomNode read,TreeItem* parent)
{
	QDomNode m = read.firstChild();
	QDomElement elem;
	TreeItem* newItem;
	QList<QVariant> blockdata;
	blockdata << 0 << 0 << 0 << 0; //initialize array to minimum size 4
	while(!m.isNull())
	{
		elem = m.toElement();
		if(elem.tagName() == "name") {
			blockdata[0] = blockdata[3] = elem.text();
			blockdata[1] = 0; // always initialize number of keys to 0
		}
		if(elem.tagName() == "displayname") {
			blockdata[0] = elem.text();
		}
		if(elem.tagName() == "enabled")
		{
			if(elem.text() == "true")
				blockdata[2] = true;
			else
				blockdata[2] = false;
			newItem = new TreeItem(blockdata,parent);
			parent->appendChild(newItem);
		}
		if(elem.tagName() == "block") goIntoBlock(m,newItem);
		if(elem.tagName() == "key") goIntoKey(m,newItem);
		m = m.nextSibling();
	}
}

void TreeModel::goIntoKey(QDomNode read,TreeItem* parent)
{
	QDomNode m = read.firstChild();
	QDomElement elem;
	QMap<QString,QVariant> key;
	while(!m.isNull())
	{
		elem = m.toElement();
		if(elem.tagName() == "name") {
			key["name"] = elem.text();
			if(!key.contains("displayname")) {
				key["displayname"] = elem.text();
			}
		}
		if(elem.tagName() == "enabled") key.insert("enabled",elem.text());
		if(elem.tagName() == "type") {
			// small optimization
			QString text = elem.text();
			if(QString("text") == text) {
				key.insert("type", 0);
			}
			if(QString("combobox") == text) {
				key.insert("type", 1);
			}
			if(QString("spinbox") == text) {
				key.insert("type", 2);
				key.insert("defaultmin",0); // set defaults for min and max
				key.insert("defaultmax",100);
				key.insert("min", 0);
				key.insert("max", 100);
			}
			if(QString("file") == text) {
				key.insert("type", 3);
			}

		}
		if(elem.tagName() == "displayname")	{
			key["displayname"] = elem.text();
		}
		if(elem.tagName() == "defaultlist")	{
			QDomNode n = m.firstChild();
			QDomElement f;
			QMap<QString, QVariant> defaultlist;
			QString value, name;
			while(!n.isNull())
			{
				f = n.toElement();
				name = f.text();
				value = f.attribute("value", name);
				defaultlist.insert(value, name);
				n = n.nextSibling();
			}
			key.insert("defaultlist", defaultlist);
			key.insert("currentlist", defaultlist); // define default if none
													// is not given later
		}
		if(elem.tagName() == "currentlist")	{
			QDomNode n = m.firstChild();
			QDomElement f;
			QMap<QString, QVariant> currentlist;
			QString value, name;
			while(!n.isNull())
			{
				f = n.toElement();
				name = f.text();
				value = f.attribute("value", name);
				currentlist.insert(value, name);
				n = n.nextSibling();
			}
			key.insert("currentlist",currentlist);
		}
		if(elem.tagName() == "defaultmin") {
			key.insert("min",elem.text());
			key.insert("defaultmin", elem.text());
		}
		if(elem.tagName() == "defaultmax") {
			key.insert("max",elem.text());
			key.insert("defaultmax", elem.text());
		}
		if(elem.tagName() == "min") { // order is critical (sequence in XSD)
			key.insert("min",elem.text());
		}
		if(elem.tagName() == "max") {
			key.insert("max",elem.text());
		}

		if(elem.tagName() == "default") key.insert("value",elem.text());
		if(elem.tagName() == "info") key.insert("info",elem.text());
		m = m.nextSibling();
	}
	parent->itemData.append(key);
}

void TreeModel::intoBlock(QDomNode read)
{
	QDomNode m = read.firstChild();
	QDomElement e;
	QString myName;
	while(!m.isNull())
	{
		e = m.toElement();
		if(e.tagName() == "name") myName = e.text();
		if(e.tagName() == "block") intoBlock(m);
		if(e.tagName() == "key") intoKey(m,myName);
		m = m.nextSibling();
	}
}

void TreeModel::intoKey(QDomNode read,QString blockname)
{
	QDomNode m = read.firstChild();
	QDomElement e;
	QString keyname;
	while(!m.isNull())
	{
		e = m.toElement();
		if(e.tagName() == "name") keyname = e.text();
		if(e.tagName() == "dep") intoDep(m,blockname,keyname);
		m = m.nextSibling();
	}
}

void TreeModel::intoDep(QDomNode read,QString blockname,QString keyname)
{
	QString trigger, target;
	trigger = blockname;
	trigger += "|";
	trigger += keyname;

	QMap< QString, QMap< QString, QMap<QString,QVariant> > > dependencies;
	QMap< QString, QMap< QString, QMap<QString,QVariant> > >::iterator dep_iter;

	QString value;

	QDomNode m, n, o;
	m = read.firstChild();
	QDomElement e, f, g, h;
	QMap< QString, QMap<QString,QVariant> > targets;
	e = m.toElement();
	if(e.tagName() == "value")
		value = e.text();

	m = m.nextSibling();
	e = m.toElement();
	while(!m.isNull())
	{
		e = m.toElement();
		if(e.tagName() == "target")
		{
			n = m.firstChild();
			while(!n.isNull())
			{
				f = n.toElement();
				if(f.tagName() == "targetblock") target = f.text();
				if(f.tagName() == "targetkey")
				{
					target += "|";
					target += f.text();
				}
				if(f.tagName() == "action")
				{
					QMap<QString,QVariant> actions;
					o = n.firstChild();
					while(!o.isNull())
					{
						g = o.toElement();
						if(g.tagName() == "enableblock")
							actions.insert("enableblock",g.text());
						if(g.tagName() == "enablekey")
							actions.insert("enablekey",g.text());
						if(g.tagName() == "setvalue")
							actions.insert("setvalue",g.text());
						if(g.tagName() == "setmin")
							actions.insert("setmin",g.text().toInt());
						if(g.tagName() == "setmax")
							actions.insert("setmax",g.text().toInt());
						if(g.tagName() == "insertitems")
						{
							QMap<QString, QVariant> dependencies;
							QDomNode q = o.firstChild();
							QString value, name;
							while(!q.isNull())
							{
								h = q.toElement();
								name = h.text();
								value = h.attribute("value", name);
								dependencies.insert(value, name);
								q = q.nextSibling();
							}
							actions.insert("insertitems",dependencies);
						}
						if(g.tagName() == "removeitems")
						{
							QMap<QString, QVariant> dependencies2;
							QDomNode p = o.firstChild();
							QString value, name;
							while(!p.isNull())
							{
								h = p.toElement();
								name = h.text();
								value = h.attribute("value", name);
								dependencies2.insert(value, name);
								p = p.nextSibling();
							}
							actions.insert("removeitems",dependencies2);
						}
						if(g.tagName() == "checkremove")
						{
							QList<QVariant> list;
							QDomNode q = o.firstChild();
							while(!q.isNull())
							{
								h = q.toElement();
								list.append(h.text());
								q = q.nextSibling();
								h = q.toElement();
								list.append(h.text());
								q = q.nextSibling();
							}
							actions.insert("checkremove",list);
						}
						o = o.nextSibling();
					}
					targets.insert(target,actions);
				}
				n = n.nextSibling();
			}
		}
		m = m.nextSibling();
	}
	dependencies.insert(value,targets);

	if(Dep.contains(trigger)) {
		QMap< QString, QMap< QString, QMap<QString,QVariant> > > deps;
		deps = Dep.value(trigger);
		for(dep_iter = deps.begin(); dep_iter != deps.end();
				dep_iter++) {
			dependencies.insert(dep_iter.key(), dep_iter.value());
		}
	}
	Dep.insert(trigger,dependencies);
}

QModelIndex TreeModel::getIndexOf(QString blockandkey) {
	QString block, key;
	if(blockandkey.contains("|")) {
		QStringList l = blockandkey.split("|");
		block = l.first();
		key = l.last();
		return getIndexOf(block, key);
	} else {
		return getIndexOf(blockandkey, QString());
	}
}

QModelIndex TreeModel::getIndexOf(QString block,QString key)
{

	QModelIndex i = firstIndex();
	TreeItem* item = static_cast<TreeItem*>(i.internalPointer());
	QString name = item->itemData.value(3).toString();

	while((name != block) && (name.remove(' ') != block))
	{
		i = nextIndex(i);
		if(i.isValid())
		{
			item = static_cast<TreeItem*>(i.internalPointer());
			name = item->itemData.value(3).toString();
		}
		else return QModelIndex();
	}

	if(key.isEmpty())
		return createIndex(item->row(),2,item);

	int col = 4;
	QMap<QString,QVariant> map = item->itemData.value(col).toMap();
	name = map.value("name").toString();

	while((name != key) && (name.remove(' ') != key))
	{
		col++;
		if(col < item->columnCount())
		{
			map = item->itemData.value(col).toMap();
			name = map.value("name").toString();
		}
		else return QModelIndex();
	}
	return createIndex(item->row(),col,item);
}

void TreeModel::countKeys(TreeItem* item)
{
	int i = item->columnCount() - 4;
	item->itemData.replace(1,i);
	for(int j = 0;j < item->childCount();j++) countKeys(item->child(j));
}

void TreeModel::deleteAll()
{
	while(rootItem->itemData.count() > 2)
		rootItem->itemData.removeLast();

	while(!rootItem->childItems.isEmpty())
		rootItem->childItems.removeLast();

	Dep.clear();
}

QModelIndex TreeModel::firstIndex()
{
	return createIndex(0,0,rootItem->child(0));
}

QModelIndex TreeModel::firstEnabledIndex()
{
	QModelIndex i = createIndex(0,0,rootItem->child(0));
	if(rootItem->child(0)->data(2).toBool())
		return i;
	else
		return nextEnabledIndex(i);
}

QModelIndex TreeModel::nextIndex(QModelIndex i)
{
	if(i.isValid())
	{
		TreeItem* item = static_cast<TreeItem*>(i.internalPointer());

		if(item->childCount() > 0)
			return createIndex(0,0,item->child(0));

		if(item->row() < item->rowCount() - 1)
			return createIndex(item->row() + 1,0,item->parent()->child(item->row() + 1));

		item = item->parent();
		while(item != rootItem)
		{
			if(item->row() < item->rowCount() - 1)
				return createIndex(item->row() + 1,0,item->parent()->child(item->row() + 1));
			item = item->parent();
		}
	}

	return QModelIndex();
}

QModelIndex TreeModel::nextEnabledIndex(QModelIndex i)
{
	if(i.isValid())
	{
		TreeItem* item = static_cast<TreeItem*>(i.internalPointer());

		// traversing children first
		for(int j = 0; j < item->childCount(); j++)
			if(item->child(j)->data(2).toBool())
				return createIndex(item->child(j)->row(),0,item->child(j));

		// then siblings
		for(int j = item->row() + 1; j < item->rowCount(); j++)
			if(item->parent()->child(j)->data(2).toBool())
				return createIndex(j,0,item->parent()->child(j));

		// then parents
		item = item->parent();
		while(item != rootItem)
		{

			for(int j = item->row() + 1; j < item->rowCount(); j++)
				if(item->parent()->child(j)->data(2).toBool())
					return createIndex(j,0,item->parent()->child(j));
			item = item->parent();
		}
	}

	return QModelIndex();
}

QModelIndex TreeModel::previousIndex(QModelIndex i)
{
	if(i.isValid())
	{
		TreeItem* item = static_cast<TreeItem*>(i.internalPointer());

		if(item->row() > 0)
		{
			int r = item->row();
			item = item->parent()->child(r-1);  // Nachbar darüber; checken ob dieser Kinder hat!
			while(item->childCount() > 0) // hat Kinder!
			{
				item = item->child(item->childCount()-1);  // bis zum letzten, untersten Kind
			}
			return createIndex(item->row(),0,item);
		}
		else 
		{
			item = item->parent();
			if(item != rootItem)
				return createIndex(item->row(),0,item);
		}
	}

	return QModelIndex();
}

QModelIndex TreeModel::previousEnabledIndex(QModelIndex i)
{
	if(i.isValid())
	{
		TreeItem* item;
		i = previousIndex(i);
		while(i.isValid())
		{
			item = static_cast<TreeItem*>(i.internalPointer());
			if(item->data(2).toBool())
				return i;
			i = previousIndex(i);
		}
	}

	return QModelIndex();
}

int TreeModel::lastEnabledChild(TreeItem* item)
{
	int i = -1;
	for(int j = 0; j < item->childCount(); j++)
		if(item->child(j)->data(2).toBool())
			i = j;
	return i;
}

QTreeWidgetItem* TreeModel::currentSettings()
{
	QTreeWidgetItem *root;
	root = new QTreeWidgetItem(0);
	root->setText(0,"Block / Key");
	root->setText(1,"Value");
	addToTree(rootItem,root);
	return root;
}

void TreeModel::addToTree(TreeItem *current,QTreeWidgetItem *parent)
{
	for(int i = 0; i < current->childCount(); i++)
	{
		if(current->child(i)->data(2).toBool())
		{
			QTreeWidgetItem *it;
			it = new QTreeWidgetItem(parent);
			it->setText(0,current->child(i)->data(0).toString());
			addToTree(current->child(i),it);
		}
	}
	for(int j = 4; j < current->columnCount(); j++)
	{
		QMap<QString,QVariant> map = current->data(j).toMap();
		if(map.value("enabled").toBool())
		{
			QTreeWidgetItem *key;
			key = new QTreeWidgetItem(parent);
			key->setText(0,map.value("displayname").toString());
			if(map.value("type") == 1) {
				key->setText(1,map.value("currentlist").toMap().value(map.value("value").toString()).toString());
			}else {
				key->setText(1,map.value("value").toString());
			}
		}
	}
}

void TreeModel::setupModelData(QDomDocument InDoc)
{
	QDomElement DocElem = InDoc.documentElement();
	QDomNode n = DocElem.firstChild();
	QDomElement e;

	while(!n.isNull())
	{
		e = n.toElement();
		if(e.tagName() == "block") goIntoBlock(n,rootItem);
		n = n.nextSibling();
	}

	for(int i = 0; i < rootItem->childCount(); i++) countKeys(rootItem->child(i));

	// create the dependency map
	n = DocElem.firstChild();

	while(!n.isNull())
	{
		e = n.toElement();
		if(e.tagName() == "block") intoBlock(n);
		n = n.nextSibling();
	}

	emit modelSetupFinished();
}

void TreeModel::disableAllBlocks(TreeItem* item)
{
	QString blockname = item->data(3).toString();
	setBlockEnabled(getIndexOf(blockname,QString()),false);
	/*QMap<QString,QVariant> map;
	  for(int i = 3; i < item->columnCount(); i++)
	  {
	  map = item->data(i).toMap();
	  map.insert("enabled",false);
	  setData(getIndexOf(blockname,map.value("name").toString()),map);
	  }*/
	for(int j = 0; j < item->childCount(); j++)
	{
		disableAllBlocks(item->child(j));
	}
}

void TreeModel::setParam(QDomDocument doc)
{
	for(int i = 0; i < rootItem->childCount(); i++)
	{
		disableAllBlocks(rootItem->child(i));
	}
	QDomElement docelem = doc.documentElement();
	QDomNode n = docelem.firstChild();
	while(!n.isNull())
	{
		loadSettings(n);
		n = n.nextSibling();
	}
}

void TreeModel::loadSettings(QDomNode n)
{
	QDomElement e = n.toElement();
	QModelIndex ib = getIndexOf(e.tagName(),QString());
	if(ib.isValid())
	{
		setBlockEnabled(ib,true);
		TreeItem* item = static_cast<TreeItem*>(ib.internalPointer());
		for(int i = 4; i < item->columnCount(); i++)
		{
			QMap<QString,QVariant> map = item->data(i).toMap();
			map.insert("enabled",false);
			QModelIndex ik = createIndex(item->row(),i,item);
			setData(ik,map);
		}
		QDomNode m = n.firstChild();
		while(!m.isNull())  // check if block or key
		{
			QDomNode o = m.firstChild();
			if(o.nodeType() == QDomNode::ElementNode) // then m must be a block
				loadSettings(m);
			else  // m is a key
			{
				QDomElement f = m.toElement();
				QModelIndex ik = getIndexOf(e.tagName(),f.tagName());
				if(ik.isValid())
				{
					QMap<QString,QVariant> map = data(ik).toMap();
					map.insert("enabled",true);
					map.insert("value",f.text());
					setData(ik,map);
				}
				else {
					emit showInfo("The Export File you want to load does not fit to the current Description!\nLoad the corresponding Description first!");
				}
			}
			m = m.nextSibling();
		}
	}
	else {
		emit showInfo("The Export File you want to load does not fit to the current Description!\nLoad the corresponding Description first!");
	}
}

void TreeModel::checkAllBlocks()
{
	for(int i = 0; i < rootItem->childCount(); i++)
		checkBlock(rootItem->child(i));
}

void TreeModel::checkBlock(TreeItem* item)
{
	QModelIndex ix = createIndex(item->row(),2,item);
	emit blockChanged(ix);
	for(int i = 0; i < item->childCount(); i++)
		checkBlock(item->child(i));
}

void TreeModel::checkDep(QString block,QString key)
{
	QString trigger = block;
	trigger += "|";
	trigger += key;
	QMap< QString, QMap< QString, QMap< QString, QMap< QString, QVariant > > > >::iterator trigger_iter;
	QMap< QString, QMap< QString, QMap< QString, QVariant > > >::iterator value_iter;
	QMap< QString, QMap< QString, QVariant > >::iterator target_iter;
	QList< QString > targetlist;

	if(Dep.contains(trigger)) /* We have to reevaluate the deps */
	{
		for(trigger_iter = Dep.begin(); trigger_iter != Dep.end();
				trigger_iter++) {
			for(value_iter = trigger_iter.value().begin(); value_iter !=
					trigger_iter.value().end(); value_iter++) {
				targetlist.append(value_iter.value().keys());
			}
		}
		QSet< QString > tmp;
		tmp = targetlist.toSet();
		targetlist = tmp.toList();

		/*
		 * For each target, iterate over the dependencies and apply the
		 * relevant ones
		 */
		for(QList< QString >::const_iterator target = targetlist.constBegin();
				target != targetlist.constEnd(); target++) {
			applyAllDepActions(*target);
		}

	}
}

void TreeModel::applyAllDepActions(QString target) {
	QMap< QString, QMap< QString, QMap< QString, QMap< QString, QVariant > > > >::iterator triggerToValue;
	QMap< QString, QMap< QString, QVariant > >::iterator targetToAction;
	QMap<QString, QMap<QString, QMap<QString,QVariant> > > valuesToTargets;
	QMap< QString, QMap<QString,QVariant> > targetsToActions;
	QMap<QString,QVariant> actions;


	QModelIndex triggerIndex;
	TreeItem *triggerItem, *targetItem;
	QMap<QString,QVariant> triggerData, targetData;
	QString triggerValue;

	targetItem = static_cast<TreeItem*>(getIndexOf(target).internalPointer());
	targetData = targetItem->data(getIndexOf(target).column()).toMap();

	QMap<QString, QVariant> currentList = targetData.value("defaultlist").toMap();
	int min, max;
	min = targetData.value("defaultmin").toInt();
	max = targetData.value("defaultmax").toInt();

	for(triggerToValue = Dep.begin(); triggerToValue != Dep.end();
			triggerToValue++) {
		triggerIndex = getIndexOf(triggerToValue.key());
		triggerItem = static_cast<TreeItem*>(triggerIndex.internalPointer());
		triggerData = triggerItem->data(triggerIndex.column()).toMap();
		triggerValue = triggerData.value("value").toString();
		valuesToTargets = triggerToValue.value();

		if(valuesToTargets.contains(triggerValue)) {
			// check if triggerValue has actions associated
			targetsToActions = valuesToTargets.value(triggerValue);
			if(targetsToActions.contains(target)) {
				// execute the actions for our target
				actions = targetsToActions.value(target);
				for(QMap<QString, QVariant>::const_iterator i =
						actions.constBegin(); i != actions.constEnd(); i++) {

					if(getIndexOf(target).column() <= 2 && i.key() ==
							"enableblock") { // this is a block
						if(i.value().toString() == "true")
							setBlockEnabled(getIndexOf(target), true);
						else
							setBlockEnabled(getIndexOf(target), false);
					}
					if(i.key() == "enablekey") {
						if(i.value() != targetData.value("enabled")) {
							targetData.insert("enabled", i.value());
						}
					}

					if(i.key() == "insertitems") {
						QMap<QString, QVariant> insertList = i.value().toMap();

						for(QMap<QString, QVariant>::const_iterator j = insertList.constBegin(); j != insertList.constEnd(); j++) {
							if(!currentList.contains(j.key()))
								currentList.insert(j.key(), j.value().toString());
						}
					}

					if(i.key() == "removeitems") {
						QMap<QString, QVariant> removeList = i.value().toMap();

						for(QMap<QString, QVariant>::const_iterator j = removeList.constBegin(); j != removeList.constEnd(); j++) {
							if(currentList.contains(j.key()))
								currentList.remove(j.key());
						}
					}

					if(i.key() == "setvalue")
					{
						if(targetData.value("value") != i.value())
						{
							targetData.insert("value",i.value());
						}
					}

					if(i.key() == "setmin")
					{
						if(min != i.value())
						{
							min = i.value().toInt();
						}
					}

					if(i.key() == "setmax")
					{
						if(max != i.value())
						{
							max = i.value().toInt();
						}
					}
				}
			}
		}
	}

	if(targetData.contains("currentlist")) {
		targetData.insert("currentlist", currentList);
	}
	if(targetData.contains("min")) {
		targetData.insert("min", min);
	}
	if(targetData.contains("max")) {
		targetData.insert("max", max);
	}
	setDataNoDeps(getIndexOf(target),targetData,Qt::EditRole);
}


void TreeModel::exportBlock(QDomNode parent,TreeItem* block)
{
	if(block->data(2).toBool()) // falls eingeschaltet
	{
		QDomNode name = QDomDocument().createElement(block->itemData.value(3).toString().remove(' '));
		parent.appendChild(name);

		for(int j = 3; j < block->columnCount(); j++)
		{
			QMap<QString,QVariant> map = block->data(j).toMap();
			if(map.value("enabled").toBool())   // falls eingeschaltet
			{
				QDomNode key = QDomDocument().createElement(map.value("name").toString().remove(' '));
				name.appendChild(key);
				QDomText t = QDomDocument().createTextNode(map.value("value").toString());
				key.appendChild(t);
			}
		}

		for(int i = 0; i < block->childCount(); i++) exportBlock(name,block->child(i));
	}
}

QDomDocument TreeModel::setupOutDoc()
{
	QDomDocument OutDoc;
	QDomElement e = OutDoc.createElement("Param");
	OutDoc.appendChild(e);

	for(int i = 0; i < rootItem->childCount(); i++) exportBlock(e,rootItem->child(i));
	return OutDoc;
}
